

/*
 * L12_Display_Temp_Humidity_Pressure_Light.ino
 * Output the Temperature, Humidity, Pressure 
 * and light value to the I2C Display.
 * 
 * FRSEF Crash Course V1 Lesson 12
 * Written by Chris Bergsneider 
 * cbergsneider@flintsciencefair.org
 * December 11, 2020
 */

// Include time libraries
#include <TimeLib.h>
#include <TimeAlarms.h>
#include <Wire.h>
#include <DS1307RTC.h>

// Display Library
#include <U8x8lib.h>

// Pressure Sensor Library
#include <Seeed_BMP280.h>

// include Adafruits DHT Digital Humidity and Temperature libraries
#include <DHT.h> 
#include <DHT_U.h>

#define DHTTYPE DHT11 // DHT11 sensor type (the blue one)

// Display Constructor
U8X8_SSD1306_128X64_ALT0_HW_I2C Display(/* reset=*/ U8X8_PIN_NONE); 

const byte dht11Pin = 3; // DHT Sensor is on pin D3
const byte lightPin = A6; // Light sensor is on pin A6

const unsigned int interval = 2000; // interval between readings in ms DHT11 is slow and it is suggested to read it at 2s intervals

DHT dht11(dht11Pin, DHTTYPE); // create an instance of the DHT class to interact with the DHT11 sensor

BMP280 bmp280;

// Convert from Celsius to Fahrenheit
float CtoF(float tempC)
{
  return tempC * 1.8 + 32;
}

void setup(void)
{
  // Start the Serial Port
  Serial.begin(9600);

  // sync the time with the RTC
  Serial.print("Syncing with RTC... ");
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  if(timeStatus()!= timeSet)
  { 
     Serial.println("Unable to sync with the RTC");
  }
  else
  {
//    stringDateTime();
//    Serial.println(sDateTime);
  }

  // Start the Display
  Display.begin();
  Display.setFlipMode(1);

  dht11.begin(); // Start the DHT11 temperature and Humidity sensor  

  bmp280.init();

}

void loop(void)
{
  static unsigned long nextTime = 0; // at what time should we re-run the code
  unsigned long ms = millis(); // get the current time
  
  if (ms >= nextTime)
  {
    nextTime = ms + interval; 

    long timeNow = now();
    
    float temperatureDHT = dht11.readTemperature(); // get the temperature already scaled in Celcius
    float humidity = dht11.readHumidity();  // get the humidity already scaled in %RH (relative humidity)

    float temperatureBMP = bmp280.getTemperature(); // get the temperature already scaled in Celcius
    float pressure = bmp280.getPressure(); // get the pressure in Pascals
    
    int lightValue = analogRead(lightPin);  //read the light sensor
    
//    Serial.print(temp);  // send the temperature in °C
//    Serial.print('\t'); // separate the values with a TAB character
//    Serial.print(CtoF(temp));  // send the temperature °F
//    Serial.print('\t'); // separate the values with a TAB character
//    Serial.println(humd); // send the humidity
    
    Display.setFont(u8x8_font_chroma48medium8_r);
    //Display.clear(); // prevent extraneous characters
    
    // Display Date
    Display.setCursor(6, 0); // set the cursor to the characters that could be short
    Display.print("  "); // clear these characters
    Display.setCursor(0, 0);
    Display.print(month(timeNow));
    Display.print('/');
    Display.print(day(timeNow));
    Display.print('/');
    int tm = year(timeNow) % 100;
    if(tm < 10)
    {
      Display.print('0');
    }
    Display.print(tm);

    // Display Time
    Display.setCursor(9, 0);
    tm = hourFormat12(timeNow);
    if(tm < 10)
    {
      Display.print(' ');
    }
    Display.print(tm);
    Display.print(':');
    tm = minute(timeNow);
    if(tm < 10)
    {
      Display.print('0');
    }
    Display.print(tm);
    if(isAM())
    {
      Display.print("AM");
    }
    else
    {
      Display.print("PM");
    }

    // Display Temperature
    Display.setCursor(5, 1); // set the cursor to the characters that could be short
    Display.print("  "); // clear these characters
    Display.setCursor(0, 1);
    //Display.print("0123456789ABCDEF");
    Display.print(temperatureBMP, 1);
    Display.print(" C");

    // Display Humidity
    Display.setCursor(14, 1); // set the cursor to the characters that could be short
    Display.print("  "); // clear these characters
    Display.setCursor(10, 1);
    Display.print(humidity, 0);
    Display.print("%RH");

    // Display Pressure
    Display.setCursor(7, 2); // set the cursor to the characters that could be short
    Display.print(' '); // clear these characters
    Display.setCursor(0, 2);
    Display.print(pressure/1000, 1);
    Display.print("kPa");

    // Display Light
    Display.setCursor(14, 2); // set the cursor to the characters that could be short
    Display.print("  "); // clear these characters
    Display.setCursor(12, 2);
    Display.print(lightValue);
  }
}
